
package aspect01;

import common.SliderListener;
import java.awt.BasicStroke;
import java.awt.Color;
import java.awt.Dimension;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.Rectangle;
import java.awt.RenderingHints;
import java.util.Observable;
import java.util.Observer;
import javax.swing.JComponent;

/**
 *
 * @author Projekte
 */
public class DrawingPanel extends JComponent implements Observer {
    /**
     * Eine Instanz von DrawingPanel herstellen und bei einem SliderListener
     * als Observer registrieren.
     * @param sl Instanz von SliderListener, bei dem die neu erzeugte
     * Instanz als Observer registriert wird
     * @return neu erzeugte Instanz von DrawingPanel
     * @see SliderListener
     */
    public static DrawingPanel createInstance(SliderListener sl) {
        DrawingPanel p = new DrawingPanel();
        p.strokeWidth = sl.getCurrentValue();
        sl.addObserver(p);
        return p;
    }
    /**
     * Die Strichstärke, mit der Linien gezeichnet werden sollen
     */
    int strokeWidth = 1;

    /**
     * Mit einem Aufruf dieser Methode teilt der beobachtete SliderListener eine
     * Änderung des eingestellten Wertes mit.
     * @param source
     * @param arg
     */
    @Override
    public void  update(Observable source,
                       Object arg) {
        strokeWidth = ((SliderListener)source).getCurrentValue();
        this.repaint();
    }

    /**
     * Methode für das Zeichnen in das Anzeigeelement. Für den eigentlichen
     * Zeichenvorgang wird die Methode <b>InhaltZeichnen</b> aufgerufen.
     * @param g Instanz von Graphics; das zu verwendende Zeicheninstrument
     */
    @Override
    public void paintComponent(Graphics g) {
        super.paintComponent(g);
        Dimension groesse = this.getSize();
        Rectangle bereich = new Rectangle(groesse);
        this.InhaltZeichnen((Graphics2D)g, bereich);
    }

    /**
     * Diese Methode erledigt den eigentlichen Zeichenvorgang
     * @param g Graphics2D, das gebrauchsfertig cvorbereitete Zeicheninstrument
     * @param bereich Rechtangle, der Bereich in den das Zeicheninstrument
     * zeichnen kann.
     */
    private void InhaltZeichnen(Graphics2D g, Rectangle bereich){
        bereich.grow(-60, -60);
        g.setColor(new Color(255, 0, 0));
        RenderingHints einstellungen = new RenderingHints(
            RenderingHints.KEY_ANTIALIASING,
            RenderingHints.VALUE_ANTIALIAS_ON);
        g.addRenderingHints(einstellungen);
        g.setStroke(new BasicStroke(strokeWidth,
                    BasicStroke.CAP_ROUND,
                    BasicStroke.JOIN_ROUND));
        g.drawLine(bereich.x, bereich.y,
               bereich.x + bereich.width - 1,
               bereich.y + bereich.height - 1);
        if (strokeWidth > 2)
        {
            g.setColor(Color.white);
            g.setStroke(new BasicStroke(1));
            g.drawLine(bereich.x, bereich.y,
               bereich.x + bereich.width - 1,
               bereich.y + bereich.height - 1);
        }
    }
}

