
package aspect02;

import java.awt.BasicStroke;
import java.awt.Color;
import java.awt.Dimension;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.Rectangle;
import java.awt.RenderingHints;
import java.util.Observable;
import java.util.Observer;
import javax.swing.JList;
import javax.swing.JComponent;
import javax.swing.event.ListSelectionEvent;

/**
 *
 * @author Projekte
 */
public class DrawingPanel2 extends JComponent implements Observer {

    private AspectSelectionListener listener1,
            listener2;
    private int strokeWidth = 15;
    private int endProperty, joinProperty;

    public DrawingPanel2 (AspectSelectionListener al1,
            AspectSelectionListener al2,
            int sel1,
            int sel2) {
        listener1 = al1;
        listener2 = al2;
        endProperty = sel1;
        joinProperty = sel2;
        this.registerAsObserver(listener1, listener2);
    }

    private void registerAsObserver(AspectSelectionListener al1, AspectSelectionListener al2) {
        al1.addObserver(this);
        al2.addObserver(this);
    }
    @Override
    public void update(Observable obj, Object arg) {
        AspectSelectionListener al = (AspectSelectionListener) obj;
        ListSelectionEvent event = (ListSelectionEvent) arg;
        JList<?> sender = (JList<?>) event.getSource();
        int[] indices = sender.getSelectedIndices();
        if (indices.length > 0) {
            if (al == listener1) {
                endProperty = ((LineEndAspect)sender.getModel().getElementAt(indices[0])).getValue();
            } else {
                joinProperty = ((LineJoinAspect)sender.getModel().getElementAt(indices[0])).getValue();
            }
            this.repaint();
        }
    }
    /**
     * Methode für das Zeichnen in das Anzeigeelement. Für den eigentlichen
     * Zeichenvorgang wird die Methode <b>InhaltZeichnen</b> aufgerufen.
     * @param g Instanz von Graphics; das zu verwendende Zeicheninstrument
     */
    @Override
    public void paintComponent(Graphics g) {
        super.paintComponent(g);
        Dimension groesse = this.getSize();
        Rectangle bereich = new Rectangle(groesse);
        this.InhaltZeichnen((Graphics2D)g, bereich);
    }

    /**
     * Diese Methode erledigt den eigentlichen Zeichenvorgang
     * @param g Graphics2D, das gebrauchsfertig cvorbereitete Zeicheninstrument
     * @param bereich Rechtangle, der Bereich in den das Zeicheninstrument
     * zeichnen kann.
     */
    private void InhaltZeichnen(Graphics2D g, Rectangle bereich){
        bereich.grow(-30, -30);
        //  Speicherplatz für die Koordinaten der
        //  Punkte des Streckenzuges bereitstellen:
        int [] xWerte = new int[3];
        int [] yWerte = new int[3];
        //  Punkte speichern:
        xWerte[0] = bereich.x;
        yWerte[0] = bereich.y;
        xWerte[1] = bereich.x + bereich.width - 1;
        yWerte[1] = bereich.y;
        xWerte[2] = bereich.x + bereich.width/2 - 1;
        yWerte[2] = bereich.y + bereich.height - 1;

        g.setColor(new Color(0, 128, 0));
        RenderingHints einstellungen = new RenderingHints(
            RenderingHints.KEY_ANTIALIASING,
            RenderingHints.VALUE_ANTIALIAS_ON);
        g.addRenderingHints(einstellungen);
        g.setStroke(new BasicStroke(strokeWidth,
                    endProperty, //BasicStroke.CAP_ROUND,
                    joinProperty)); //BasicStroke.JOIN_ROUND));
        g.drawPolyline(xWerte, yWerte, 3);
        if (strokeWidth > 2)
        {
            g.setColor(Color.white);
            g.setStroke(new BasicStroke(1));
            g.drawPolyline(xWerte, yWerte, 3);
        }
    }
}
