
package aspect04;

import common.RemoveableCheckBox;
import java.awt.BasicStroke;
import java.awt.Color;
import java.awt.Dimension;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.Rectangle;
import java.awt.RenderingHints;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.geom.GeneralPath;
import java.awt.geom.Path2D;
import javax.swing.JComboBox;
import javax.swing.JComponent;

/**
 *
 * @author Projekte
 */
public class DiagramPainter extends JComponent
         implements ActionListener {

    private JComboBox<Integer> valueSelectionSource1;
    private JComboBox<Integer> valueSelectionSource2;
    private RemoveableCheckBox useAntialiasingSource;
    private int param1 = 0;
    private int param2 = 0;
    private boolean useAntialiasing;

    public DiagramPainter (JComboBox<Integer> cb1,
            JComboBox<Integer> cb2,
            RemoveableCheckBox option) {
        valueSelectionSource1 = cb1;
        valueSelectionSource2 = cb2;
        useAntialiasingSource = option;
        if (valueSelectionSource1.getModel().getSelectedItem() != null) {
            param1 = (int)valueSelectionSource1.getModel().getSelectedItem();
        }
        if (valueSelectionSource2.getModel().getSelectedItem() != null) {
            param2 = (int)valueSelectionSource2.getModel().getSelectedItem();
        }
        useAntialiasing = useAntialiasingSource.getModel().isSelected();
    }

    @Override
    public void actionPerformed(ActionEvent e) {
        Object source = e.getSource();
        if (source == valueSelectionSource1) {
            param1 = (int)valueSelectionSource1.getSelectedItem();
        } else if (source == valueSelectionSource2) {
            param2 = (int)valueSelectionSource2.getSelectedItem();
        } else {
            useAntialiasing = useAntialiasingSource.getModel().isSelected();
        }
        this.repaint();
    }

    /**
     * Methode für das Zeichnen in das Anzeigeelement. Für den eigentlichen
     * Zeichenvorgang wird die Methode <b>InhaltZeichnen</b> aufgerufen.
     * @param g Instanz von Graphics; das zu verwendende Zeicheninstrument
     */
    @Override
    public void paintComponent(Graphics g) {
        super.paintComponent(g);
        Dimension groesse = this.getSize();
        Rectangle bereich = new Rectangle(groesse);
        this.InhaltZeichnen((Graphics2D)g, bereich);
    }

    /**
     * Diese Methode erledigt den eigentlichen Zeichenvorgang
     * @param g Graphics2D, das gebrauchsfertig cvorbereitete Zeicheninstrument
     * @param bereich Rechtangle, der Bereich in den das Zeicheninstrument
     * zeichnen kann.
     */
    private void InhaltZeichnen(Graphics2D g, Rectangle bereich){
        if ((param1 == 0) || (param2 == 0)) {
            return;
        }
        final int numberOfPoints = 600;
        final double delta = Math.PI*2 / numberOfPoints;
        boolean firstPt = true;
        Rectangle r = new Rectangle(bereich);
        r.grow(-10, -10);
        g.setColor(new Color(220, 0, 0));
        g.setStroke(new BasicStroke(0.5f));
        RenderingHints einstellungen = new RenderingHints(
            RenderingHints.KEY_ANTIALIASING,
            useAntialiasing ? RenderingHints.VALUE_ANTIALIAS_ON
                : RenderingHints.VALUE_ANTIALIAS_OFF);
        g.addRenderingHints(einstellungen);
        GeneralPath path = new GeneralPath(Path2D.WIND_NON_ZERO, numberOfPoints);
        for(int i = 0; i < numberOfPoints; i++) {
            double t, x, y, xt,yt;
            t = delta*i;
            x = Math.sin(param1*t);
            y = Math.cos(param2*t);
            //x = Math.sin(param1*t);
            //y = Math.sin(param2*t - Math.PI * 1.0 / 4.0);
            xt = (x + 1) * r.width / 2.0  + r.x;
            yt = (1 - y) * r.height / 2.0 + r.y;
            if (firstPt) {
                path.moveTo(xt, yt);
                firstPt = false;
            } else {
                path.lineTo(xt, yt);
            }
        }
        path.closePath();
        g.draw(path);

    }


}
