
package mvc002;

import mvc.SimpleModel;
import mvc.ComponentContainer;
import java.awt.Component;
import java.awt.Container;
import java.awt.Dimension;
import java.awt.Insets;
import java.awt.LayoutManager;
import javax.swing.ButtonGroup;
import javax.swing.JFrame;
import javax.swing.JToggleButton;

/**
 *
 * @author Projekte
 */
public class MainWindow implements Runnable, LayoutManager {

    private ApplicationModel model;

    public MainWindow (ApplicationModel am) {
        model = am;
    }

    @Override
    public void run() {
        this.createAndShowGUI();
    }

     /**
     * Herstellung des Hauptfensters der Anwendung.
     */
    private void createAndShowGUI()
    {
        JFrame frame = new JFrame("MVC - First Experiment");
        frame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
        addComponentsToPane(frame.getContentPane());

        frame.pack();
        frame.setSize(400, 400);
        frame.setLocationRelativeTo(null);
        frame.setVisible(true);
    }

    /**
     * Diese Methode fügt Anzeige- und Bedienelemente
     * in das Fenster ein.
     * @param  container eine Instanz von Container
     *
     */
    private void addComponentsToPane (Container container)
    {
        container.setLayout(this);
        ButtonGroup buttonGroup = new ButtonGroup();
        JToggleButton bttn1 = new JToggleButton("Button 1");
        JToggleButton bttn2 = new JToggleButton("Button 2");
        JToggleButton bttn3 = new JToggleButton("Button 3");
        JToggleButton bttn4 = new JToggleButton("Button 4");
        ComponentContainer<SimpleModel> componentContainer;
        componentContainer =
                new ComponentContainer<>(model.getModelHolder());

        bttn1.addActionListener(new ButtonChangeAdapter(model, Aspect.ITEM_01));
        bttn2.addActionListener(new ButtonChangeAdapter(model, Aspect.ITEM_02));
        bttn3.addActionListener(new ButtonChangeAdapter(model, Aspect.ITEM_03));
        bttn4.addActionListener(new ButtonChangeAdapter(model, Aspect.ITEM_04));

        buttonGroup.add(bttn1);
        buttonGroup.add(bttn2);
        buttonGroup.add(bttn3);
        buttonGroup.add(bttn4);

        container.add(bttn1);
        container.add(bttn2);
        container.add(bttn3);
        container.add(bttn4);
        container.add(componentContainer);

    }

    @Override
    public void addLayoutComponent(String name, Component comp) {
        // no action needed
    }

    @Override
    public void removeLayoutComponent(Component comp) {
        // no action needed
    }

    @Override
    public Dimension preferredLayoutSize(Container parent) {
        Dimension dim = new Dimension(0, 0);

        //Always add the container's insets!
        Insets insets = parent.getInsets();
        dim.width = 100
                    + insets.left + insets.right;
        dim.height = 100
                     + insets.top + insets.bottom;

        return dim;
    }

    @Override
    public Dimension minimumLayoutSize(Container parent) {
        Dimension dim = new Dimension(0, 0);

        //Always add the container's insets!
        Insets insets = parent.getInsets();
        dim.width = 60
                    + insets.left + insets.right;
        dim.height = 26
                     + insets.top + insets.bottom;

        return dim;
    }

    @Override
    public void layoutContainer(Container parent) {
        Insets insets = parent.getInsets();
        int maxWidth = parent.getWidth()
                       - (insets.left + insets.right);
        int maxHeight = parent.getHeight()
                        - (insets.top + insets.bottom);
        int nComps = parent.getComponentCount();
        int x;
        int y = insets.top;
        int w;
        final int buttonHeight= 26;
        // now, compute x origin and width for all four components
        for (int i = 0; i < 4; i++) {
            Component c = parent.getComponent(i);
            w = maxWidth * (i + 1) / 4 - maxWidth * i/4;
            x = maxWidth * i / 4;
            c.setBounds(x, y, w, buttonHeight);
        }
        Component c = parent.getComponent(4);
        c.setBounds(0, 26, maxWidth, maxHeight - buttonHeight);
    }
}
