package mvc002;


import aspect01.SimpleModel1;
import aspect02.SimpleModel2;
import aspect03.SimpleModel3;
import aspect04.SimpleModel4;
import java.util.Observable;
import java.util.Observer;
import mvc.EnumHolder;
import mvc.IEnumIDs;
import mvc.IModel;
import mvc.ModelHolder;
import mvc.SimpleModel;


/**
 * The instance manages all models that survive the lifetimes of their
 * presenters. Upon first use, these models are lazily created here.
 * @author Projekte
 */
public class ValueChannel<A extends IEnumIDs, Type extends IModel<Type>>
         implements Observer {

    ModelHolder<Type> modelHolder;
    SimpleModel mm1;
    SimpleModel mm2;
    SimpleModel mm3;
    SimpleModel mm4;

    public ValueChannel(EnumHolder<A> vh, ModelHolder<Type> holder) {
        modelHolder = holder;
        this.registerAsObserver(vh);
    }

    private void registerAsObserver(EnumHolder<A> vh) {
        vh.addObserver(this);
    }

    /**
     * Der einzige Zweck dieser Methode ist, die Zahl der vom Compiler
     * angegebenen Warnungen (unchecked cast) auf eine zu reduzieren.
     * @param m die Instanz, die in den ModelHolder eingetragen werden soll.
     */
    private void storeInModelHolder(SimpleModel m) {
        modelHolder.setModel((Type)m);
    }

    @Override
    public void update(Observable observable, Object arg) {
        EnumHolder<A> vh = (EnumHolder<A>) observable;

        switch (vh.getValue().getID())
        {
            case 0:
                modelHolder.setModel(null);
                break;
            case 1:
                if (mm1 == null) {
                    mm1 = new SimpleModel1();
                }
                this.storeInModelHolder(mm1);
                break;
            case 2:
                if (mm2 == null) {
                    mm2 = new SimpleModel2();
                }
                //((Type)mm2).storeIn(modelHolder);
                this.storeInModelHolder(mm2);
                break;
            case 3:
                if (mm3 == null) {
                    mm3 = new SimpleModel3();
                }
                this.storeInModelHolder(mm3);
                break;
            case 4:
                if (mm4 == null) {
                    mm4 = new SimpleModel4();
                }
                this.storeInModelHolder(mm4);
                break;
            default:
                break;
        }
    }
}
